//
// makes output file(s) so that a plot may be displayed.  the
// plot will contain both the theoretical curve f(x) vs x, and,
// superposed on it, the data points.  the scheme used to make
// the points for f(x) vs x is to make a number of points that
// will be joined by short straight-line segments so the overall
// graph looks like a smooth curve.  the separation dx between
// adjacent points is adjusted so that the absolute value of the
// difference between f(x+dx) and the tangent to f(x) at x is
// just less than a small fraction epsy of the overall range in
// y, where y = f(x).  files suitable for display using either
// goplot or gnuplot will be formed, depending on which option
// is chosen in the "fit" command line.
//

#include "fit.h"

void mk_outfile(dbl* x, dbl* y, dbl* s, dbl* b, misc* mp,\
                flags* fp, int npts)
{
    dbl x_range, y_range;

//
// first, get the date into "date"
//

    char     date[maxword];
    time_t now = time(0);
    time_t strft = strftime(date, maxword, "at %l:%M %p on %b %d, %Y.",
                      localtime(&now));
    gassert(strft);            // strft = 0 if maxword is too small
//
// then open "datafile.out" and print the header
//
    string ofn = mp->ofile[0];  // ofile[0] is "datafile.out"
    string tmp = chop_ext(ofn); // tmp is "datafile"
    mp->ofs.open(ofn.c_str());  // opens "datafile.out"
    gassert(mp->ofs);
    mp->ofs <<
        "#############################################\n"
        "# an attempted fit to data in \'" << tmp << "\' by a\n"
        "# " << mp->npar << "-parameter function of " << mp->title << ".\n"
        "# the fit was performed " << date << "\n"
        "#############################################\n";

//  
// print x and f(x) values in datafile.out
//  

    func_pts(x, y, b, npts, mp);
   
    x_range = mp->lm[1] - mp->lm[0]; // lm[k] are xmin, xmax,
    y_range = mp->lm[3] - mp->lm[2]; //           ymin, ymax.
 
//  
//  if goplot, append original data to datafile.out
//  
    if (fp->o == goplot) {
        int j = mp->prec + 10;  // used for field width below
        mp->ofs.fill(' ');
        mp->ofs.setf(ios::left);
        mp->ofs.setf(ios::floatfield);
        mp->ofs.setf(ios::showpoint);
        mp->ofs <<
            "$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$\n"; // separates data pts
        for (int i = 0 ; i < npts ; i++) {
            if (fp->s) {       // include s[i] only if "absolute"
                               // i.e., if "sigma" option is set
                mp->ofs << setw(j) << x[i]
                        << setw(j) << y[i]
                        << setw(j) << s[i] << endl;
                
            }
            else {
                mp->ofs << setw(j) << x[i]
                        << setw(j) << y[i] << endl;
            }
        }
    }
    mp->ofs.close();  // close datafile.out
  
//
// make datafile.gnu for gnuplot scripts
//  

    if (fp->o == gnuplot || fp->o == gnups) {
        ofn = mp->ofile[1];        // ofile[1] is "datafile.gnu"
        tmp = chop_ext(ofn);
        mp->ofs.open(ofn.c_str()); // open "datafile.gnu"
        gassert(mp->ofs);
 
        if (fp->o == gnuplot) {
            mp->ofs << 
              "# gnuplot script to display a graph on the terminal screen\n\n"
              "# uncomment commands below if you want them;\n"
              "# only the essential ones are activated here\n\n"
              "# this works for xterms:\n"
              "set term x11\n\n"
              "# if you like the tek display (see also below):\n"
              "# set term vttek\n\n"
              "# for some terminals this works\n"
              "# (you may not need that weird character string):\n"
              "# set term tek40xx\n"
              "# pause 0 \"\\033[\?38h\"\n"
              << "\n";
        }
        if (fp->o == gnups) {
            tmp += ".eps"; // "datafile.eps" is name of postscript file
            mp->ofs <<
              "# gnuplot script to produce an encapsulated postscript file\n\n"
              "# uncomment commands below if you want them;\n"
              "# only the essential ones are activated here\n\n"
              "# this says make encapsulated postscript file;\n"
              "# alter the font and point size to suit:\n"
              "set term post eps enhanced \"Times-Bold\" 22\n\n"
              "# this says put output in \'" << tmp << "\'\n"
              "set output \"" << tmp << "\"\n"
              << "\n";
        }
        mp->ofs <<  // both gnuplot and gnups
          "# comment out the next line if you want a key on your graph:\n"
          "set nokey\n\n"
          "# insert labels on the x and y axes if you want:\n"
          "# set xlabel \"Energy (channel number)\"\n"
          "# set ylabel \"Counts\"\n\n"
          "# if you want a title ABOVE the graph:\n"
          "# set title \"from the X-ray fluorescence experiment\"\n\n"
          "# or add a title ON the graph if you want one:\n"
          "# set label \"One gaussian line\" at 8, 1500\n\n"
          "# superpose a grid on your plot if you want:\n"
          "# set grid\n\n"
          "# tne next line plots your data and your theoretical function:"
          << "\n";

        mp->lm[0] -= 0.05 * x_range; // extend plot x-range by 10 % ---
        mp->lm[1] += 0.05 * x_range; // 5 % at each end

        tmp = chop_ext(ofn);         // tmp is now "datafile" again

        mp->ofs <<
          "plot [" << mp->lm[0] << ":" << mp->lm[1] << "] \""
          << tmp << "\" with " ;

        if (fp->s) {                 // use errorbars if sigma is set
            mp->ofs << "errorbars,\\\n\""
            << mp->ofile[0] << "\" with lines\n\n";
        }
        else {
            if (fp->o == gnuplot) {  // boxes are different pt for gnups
                mp->ofs << "points pt 3 ps 1.5,\\\n\""
                << mp->ofile[0] << "\" with lines\n\n";
            }
            else if (fp->o == gnups) {
                mp->ofs << "points pt 4 ps 1.5,\\\n\""
                << mp->ofile[0] << "\" with lines\n\n";
            }
            else {
                err("shouldn't get to this point in \'mk_outfile\'.");
            }
        }
        if (fp->o == gnuplot) {
            mp->ofs <<
              "# this keeps your graph on the screen so you can see it:\n"
              "pause -1 \"\\n\\t push \\\"q\\\" and "
              "\\\"return\\\" to exit...\\n\"\n\n"
              "# replace above line with this if you use vttek or tek40xx:\n"
              "# pause 0 \"\\037\\033\\003\"" << "\n";
        }
        if (fp->o == gnups) {
            tmp += ".eps";
            mp->ofs <<
              "pause 0 \"\\n\\t \140" << tmp << "\047 has been produced...\\\n"
              "\\n\\t  so now try \140gv " << tmp << "\047\\n\""<< "\n";
        }
        mp->ofs.close();
    }
    return;
}

