#include "utils.h"
#include <iomanip>

//
// graceful rewind by al kelley
//
void grewind(istream& is)
{
    is.clear();
    is.seekg(ios::beg);
    if (!is) err("call to grewind() failed");
}

//
// graceful assert
//
void ggassert(cchr* exp, int line, cchr* file, cchr* func, bool expr)
{
    cchr* failed = "\n uh-oh: \n  in ";
    try {
        if (!expr) throw failed;
    }
    catch( cchr* msg ) {
        cerr << msg << file << ": line " << line << ":\n    " << func
             << ":\n      " << "the assertion `" << exp
             << "' failed.\n bye ...\n\n";
        exit(1);
    }
}

//
// returns the logarithm of n factorial.
// from numerical recipes 2/e page 215.
// renamed to make it easier to recall.  5/16/97
//

float lnfac(int n)
{
    static float a[101];  // a static array initialized to zero

    gassert(n >= 0);
    if (n <= 1) return 0.0;
    if (n <= 100)         // in range of table
        return a[n] ? a[n] : (a[n] = lngam(n + 1.0));
    else                  // out of range of table
        return lngam(n + 1.0);
}

//
// returns ln gamma(xx) for xx > 0
// see press 2/e p213, who uses gammln for the name.
//

float lngam(float xx)
{
    dbl x, y, tmp, ser;
    static dbl cof[6] = { 76.18009172947146, -86.50532032941677,
        24.01409824083091, -1.231739572450155,
        0.1208650973866179e-2, -0.5395239384953e-5 };
    int j;

    gassert(xx > 0.0);
    y = x = xx;
    tmp = x + 5.5;
    gassert(tmp > 0.0);
    tmp -= (x + 0.5) * log(tmp);
    ser = 1.000000000190015;
    for (j = 0 ; j < 6 ; j++)
        ser += cof[j]/++y;
    return -tmp + log(2.5066282746310005*ser/x);
}

//
// returns incomplete gamma function Q(a,x) = 1.0-P(a,x),
// evaluated by its continued fraction rep, as gamcf.
// also returns ln gamma(a) as lng. from press 2/e p218
//

cint ITMAX = 100;     // maximum no of iterations
cdbl EPS = 3.0e-7;    // relative accuracy
cdbl FPMIN = 1.0e-30; // near the smallest representable float

void gcf(dbl* gamcf, dbl a, dbl x, dbl* lng)
{
    int i;
    dbl an, b, c, d, del, h;

    *lng = lngam(a);
    b = x + 1.0 - a;
    c = 1.0/FPMIN;
    gassert(b != 0.0);
    d = 1.0/b;
    h = d;
    for (i = 1; i <= ITMAX ; i++) {
        an = -i * (i - a);
        b += 2.0;
        d = an * d + b;
        if (fabs(d) < FPMIN) d = FPMIN;
        c = b + an/c;
        if (fabs(c) < FPMIN) c = FPMIN;
        gassert(d != 0.0);
        d = 1.0/d;
        del = d*c;
        h *= del;
        if (fabs(del - 1.0) < EPS)
            break;
    }
    if (i > ITMAX) {
        err("near x = ", x, ", \"a\" too large, ITMAX too "
            "small in routine gcf");
    }
    gassert(x > 0.0) ;
    *gamcf = exp(-x + a*log(x) - (*lng)) * h;
}
//
// returns incomplete gamma function P(a,x),
// evaluated by its series rep, as gamser.
// also returns ln gamma(a) as lng. from press 2/e p218
//

// cint ITMAX = 200;  // maximum no of iterations
// cdbl EPS = 3.0e-7; // relative accuracy

void gser(dbl* gamser, dbl a, dbl x, dbl* lng)
{
    int n;
    dbl sum, del, ap;

    *lng = lngam(a);
    if (x <= 0.0) {
        if (x < 0.0)
            err("x less than 0.0 in routine \"gser\"");
        *gamser = 0.0;
        return;
    } else {
        ap = a;
        del = sum = 1.0/a;
        for (n = 1 ; n <= ITMAX ; n++) {
            ++ap;
            del *= x/ap;
            sum += del;
            if (fabs(del) < fabs(sum)*EPS) {
                gassert(x > 0.0);
                *gamser = sum * exp( -x + a*log(x) - (*lng));
                return;
            }
        }
        err("\nat x = ", x, "a = ", a, "\"a\" (nu/2) too "
            "large or ITMAX too small in routine gser");
        return;
    }
}

//
//    returns incomplete gamma function
//         Q(a,x) = 1 - P(a,x)
// this is a modification of the code given
// by press 2/e page 218, to make convergence
// better. converges for 0.0 < a <= 2624 now.
// written 5/18/97
//


dbl gammq(dbl a, dbl x)
{
    dbl gamser,  // for the series rep
          gamcf,   // for the continued fraction rep
          lng;     // log of gamma(a)

    if (x < 0.0 || a <= 0.0)
        err("invalid arguments in gammq");
    if (x > 3.0) {
        if (x <= a - 1.0) {
            gser(&gamser, a, x, &lng);
            return 1.0 - gamser;
        }
        else {     // x == a-1.0 not allowed in gcf
            gcf(&gamcf, a, x, &lng);
            return gamcf;
        }
    }
    else {         // 0.0 <= x <= 3.0
        if (x < a + 1.0) {
            gser(&gamser, a, x, &lng);
            return 1.0 - gamser;
        }
        else {
            gcf(&gamcf, a, x, &lng);
            return gamcf;
        }
    }
}


array::array()
{
    md      = 0;
    vd      = 0;
    mi      = 0;
    vi      = 0;
    rows    = 0;
    cols    = 0;
    elems   = 0;
}

//
// allocate memory for a matrix of r*c doubles
// returns a pointer to the matrix
//
dbl** array::alloc_d(int r, int c)
{
//    int k;
    rows = r;
    cols = c;

    md = new dbl* [rows];
    gassert(md);

    md[0] = new dbl [rows*cols];
    gassert(md[0]);

    for ( int k=1 ; k < rows ; k++ ) {
        md[k] = md[k-1] + cols;
    }
    return md;
}
//
// allocate memory for a vector of n doubles
// returns a pointer to the vector
//

dbl* array::alloc_d(int n)
{
    elems = n;

    vd = new dbl [elems];
    gassert(vd);
    return vd;
}
//
// allocate memory for a vector of n ints
// returns a pointer to the vector
//

int* array::alloc_i(int n)
{
    elems = n;

    vi = new int [elems];
    gassert(vi);
    return vi;
}

void array::dele(dbl** mdp)
{
    delete [] mdp[0];
    delete [] mdp;
    mdp = 0;
}
void array::dele(dbl* vdp)
{
    gassert(vdp);
    delete [] vdp;
    vdp = 0;
}
void array::dele(int* vip)
{
    gassert(vip);
    delete [] vip;
    vip = 0;
}

// fills matrix with random numbers between -2500 and +2500
void array::fill_mat(dbl** mp)
{
    srand48(time(0));
    for ( int i=0 ; i<rows ; i++ ) {
        for ( int j=0 ; j<cols ; j++ ) {
            mp[i][j] = (drand48() * 5000.0) -2500.0;
        }
    }
}

// fills vector with random numbers between -2500 and +2500
void array::fill_vec(dbl* vp)
{
    for (int i = 0 ; i < elems ; i++ ) {
        vp[i] = (drand48() * 5000.0) - 2500.0;
    }
}

//
// prints a matrix of r rows and c cols on os
//

void array::pr_mat(string nam, dbl** mp, int prec, int r, int c, ostream& os)
{
    rows = r;
    cols = c;
    const ios::fmtflags os_def_flgs = os.flags(); // save default flags

    ios::fmtflags flgs = os_def_flgs
        | ios::left          // left side of field
        | ios::floatfield    // fixed or scientific
        | ios::showpoint     // shows decimal point
        | ios::showpos;      // shows + sign

    os.flags(flgs);
    os.precision(prec);
    int len = nam.length();
    os << "\n" << setw(len) << nam.c_str() << ":\n\n";

    for ( int i=0 ; i < rows ; i++ ) {
        for ( int j = 0 ; j < cols ; j++ ) {
            os << setw(prec+7) << mp[i][j];
        }
        os << "\n";
    }
    os << "\n" ;
    os.flags(os_def_flgs);  // reset to default flags
}

//
// prints lower diagonal half of a symmetric square matrix
// of r rows and r columns on os
//

void array::pr_sym_mat(string nam, dbl** mp, int prec, int r, ostream& os)
{
    rows = r;
    int m = 1;
    const ios::fmtflags os_def_flgs = os.flags(); // save default flags

    ios::fmtflags flgs = os_def_flgs
        | ios::left          // left side of field
        | ios::floatfield    // fixed or scientific
        | ios::showpoint     // shows decimal point
        | ios::showpos;      // shows + sign

    os.flags(flgs);
    os.precision(prec);
    int len = nam.length();
    cint screenwidth = 72;
    cint fields = screenwidth/(prec+7);
    if (rows > 9) m = 2;
    os << "\n" << setw(len) << nam.c_str() << "\n";

    for ( int i=0 ; i < rows ; i++ ) {
        if (rows > fields) {
            os.unsetf(ios::showpos);
            os.setf(ios::right);
            os << "row " << setw(m) << i+1 << ": ";
            os.setf(ios::showpos);
            os.setf(ios::left);
        }
        else
            os << setw(m) << "";
        for ( int j = 0 ; j <= i ; j++ ) {
            os << setw(prec+7) << mp[i][j];
            if (!((j+1)%fields) && (j != i)) {
                os << "\n";
                os << setw(m+6) << "";
            }
        }
        os << "\n";
    }
    os << "\n";
    os.flags(os_def_flgs);  // reset to default flags
}

//
// prints a vector of n doubles on os
//
void array::pr_vec(string nam, dbl* vp, int prec, int n, ostream& os)
{
    elems = n;
    cint screen_width = 79;
    const ios::fmtflags os_def_flgs = os.flags(); // save default flags
    int fields = screen_width/(prec+7);

    ios::fmtflags flgs = os_def_flgs
        | ios::left
        | ios::floatfield
        | ios::showpoint
        | ios::showpos;

    os.flags(flgs);
    os.precision(prec);     // no of sig figs
    int len = nam.length();
    os << "\n" << setw(len) << nam.c_str() << ": ";

    for ( int i=0 ; i < elems ; i++ ) {
        if (!((i+1)%fields)) {
            os << vp[i] << "\n";
            os << setw(len+2) << "";
        }
        else {
            os << setw(prec+7) << vp[i];
        }
    }
    os << "\n";
    os.flags(os_def_flgs);  // reset to default flags
}

void array::set_elems( int n )
{
     elems = n;
}
